'*------------------------------------------------------------------------------*
'* INSTRUCTION MANUAL, AS POSTED TO FORUMS:                                     *
'* Just start typing. There are 37 characters per line, and you move up and     *
'* down through the lines by using the up and down arrow keys. Right now        *
'* there's no way to move left and right through the text with the arrow keys,  *
'* and there's no way to insert text behind other text-- it just overwrites.    *
'*------------------------------------------------------------------------------*

'Written by Eddie Nolan, released under CC-by-sa 3.0
'Legal text available at http://creativecommons.org/licenses/by-sa/3.0/legalcode

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  '-----
  'The following constants are for convenience; they are the
  'numerical representations of certain keyboard input keys
  backspace = $C8
  delete = $C9
  space = $20
  enter = $0D
  pageup = $C6
  pagedown = $C7
  upkey = $C2
  downkey = $C3
  leftkey = $C0
  rightkey = $C1
  '-----
  null = $BC 'This character can be pretty much any non-alphanumerical character,
             'used to represent unprinted characters in some cases
  '-----
  'These characters represent characters in the internal parallax font
  'used for displaying scrollbars, etc.
  uparrow = $04
  downarrow = $05
  tvbackspace = $08
  block = $0E
  line = $7C
  thickline = $7F
  '-----
  clearscreen = $00 'Used with TV.out to clear screen
  '-----
  numberofcharacters = 4000 'Max number of characters that the program can store


OBJ
  KB: "Keyboard"
  TV: "TV_Text-WP"

VAR
  byte x[numberofcharacters]        'The x-value on screen for each character index number
  byte y[numberofcharacters]        'The y-value on screen for each character index number
  byte char[numberofcharacters]     'The character to display for each character index number
  word lineend[120]                 'Save the character index number at which the line ends for each line
  byte InputKey       'The key that is input via the keyboard
  byte ydisplay       'Account for hidden lines offscreen
  byte offset         'Number of lines offscreen
  byte tvvalue        'Used in KeyboardPlayback loop
  byte cursorok       'Boolean: Whether or not to go on to a new line at character 37
  word i
  word j
  word k
  word l
  word m              'Miscellaneous variables used in loops

PUB Start
  KB.Startx(26, 27, %0_000_000, %01_01000)
  TV.Start(12)
  WaitCnt(50_000_000 + cnt)
  Repeat                                                'This loop checks to see if the keyboard
    If KB.Present                                       'is present, and tells the user to plug in
                                                        'the keyboard if not

        i := 0                                                'Start the program at x=0, y=0, char=0
        bytefill(@char, null, numberofcharacters)             'Fill character buffer with 'null' characters
        lineend[0] := 0
        offset := 0                                     'Initialize variables
        TV.setxy(0,12)
        TV.str(string("WELCOME            "))           'Display welcome message (line 12 reserved for system messages)
        TV.setxy(0,0)
        DrawScrollbar(1)                                'Draw initial scrollbar
        InitXY                                          'Initiate x and y arrays
        KeyboardPlayback                                'Begin main loop
    Else
      TV.setxy(0,12)
      TV.str(string("PLUG IN KEYBOARD"))

PRI InitXY
  j := 0
  k := 0
  Repeat m from 0 to numberofcharacters
    x[m] := j
    y[m] := k
    If j == 37
      j := 0
      k += 1
    Else
      j += 1
  y[0] := 0


PRI KeyboardPlayback
  Repeat
    InputKey := KB.GetKey                               'KB.GetKey pauses program until keyboard input recieved
    InputRoutine                                        'This subroutine processes the keyboard input
    If i == 65535
      i := 0  'Prevent bug with delete key
    tvvalue := 12 + offset
    If y[i] == tvvalue
      MoveTextDown                                     'If line 12 reached, move text so that cursor can go down
    If y[i] < offset
      MoveTextUp                                        'If cursor goes offscreen, move text so cursor can go up
    TV.setxy(25, 12)                                    'The following code block displays x and y
    TV.dec(x[i])                                        'values in lower-right corner
    TV.out(",")
    TV.dec(y[i])
    TV.out(",")                                        'Uncomment this code to display variable values for debugging
    TV.dec(i)
    TV.out(",")
    TV.dec(offset)
    TV.str(string("  "))
    ydisplay := y[i] - offset
    TV.setxy(x[i], ydisplay)

PRI InputRoutine
  Case InputKey
    backspace:                  'If backspace pressed, go back a space
      i -= 1
      TV.out(space)
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)
      TV.out(block)

    delete:                     'If delete key pressed, go back a space (identical to backspace routine)
      i -= 1
      TV.out(space)
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)
      TV.out(block)

    enter:                      'If enter key pressed, go down a line
      TV.out(space)
      j := y[i] * 38
      k := i - j
      k := 38 - k
      i += k
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)
      TV.out(block)

    pageup:
      MoveTextUp

    pagedown:
      MoveTextDown

    leftkey:
      If i > 0
        If char[i] <> null
          TV.out(char[i])
        Else
          TV.out(space)
        i -= 1
        ydisplay := y[i] - offset
        TV.setxy(x[i], ydisplay)
        TV.out(block)
        DrawScrollbar(y[i])

    rightkey:
      If i > 0
        If char[i] <> null
          TV.out(char[i])
        Else
          TV.out(space)
        i += 1
        ydisplay := y[i] - offset
        TV.setxy(x[i], ydisplay)
        TV.out(block)
        DrawScrollbar(y[i])

    upkey:
      If i > 37
        If char[i] <> null
          TV.out(char[i])
        Else
          TV.out(space)
        i -= 38
        ydisplay := y[i] - offset
        TV.setxy(x[i], ydisplay)
        TV.out(block)
        DrawScrollbar(y[i])

    downkey:
      If char[i] <> null
        TV.out(char[i])
      Else
        TV.out(space)
      i += 38
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)
      TV.out(block)
      DrawScrollbar(y[i])


    OTHER:                      'If normal key pressed, print character
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)  'Set x and y
      char[i] := InputKey       'Set character for current index value to input character
      TV.out(char[i])           'Display character
      TV.out(block)             'Display cursor
      j := i
      i += 1
      'x[i] := x[j] + 1
      'y[i] := y[j]
      lineend[y[i]] := i     'Increment character index value and line-end value
      DrawScrollbar(y[i])    'Draw scrollbar

PRI MoveTextDown
  TV.out(clearscreen)
  offset += 1                   'Increase line offset
  j := offset * 38
  l := offset + 11
  l *= 38
  repeat m from j to l
    ydisplay := y[m] - offset
    TV.setxy(x[m], ydisplay)
    If char[m] <> null          'Ignore if 'null'
      TV.out(char[m])
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)
  TV.out(block)
  DrawScrollbar(y[i])        'Set x and y and draw cursor and scrollbar

PRI MoveTextUp
  TV.out(clearscreen)
  offset -= 1                   'Decrease line offset
  j := offset * 38
  l := offset + 11
  l *= 38
  repeat m from j to l
    ydisplay := y[m] - offset
    TV.setxy(x[m], ydisplay)
    If char[m] <> null          'Ignore if 'null'
      TV.out(char[m])
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)
  TV.out(block)
  DrawScrollbar(y[i])        'Set x and y and draw cursor and scrollbar

PRI DrawScrollbar (linenumber)
  TV.setxy(39,0)
  TV.out(uparrow)
  TV.setxy(39,12)
  TV.out(downarrow)
  repeat m from 1 to 11
    TV.setxy(39,m)
    TV.out(line)
  m := 0
  k := 0
  l := 0
  repeat until m == 1           'This loop finds the current line number, multiplies by 11
    If lineend[k] == 0          'and divides by the total number of lines to decide where to
      l := k + 1                'write the scrollbar character
      If lineend[l] == 0
        m := y[i] * 11
        l := m / k
        l += 1
        TV.setxy(39,l)
        TV.out(thickline)
        Return
    k += 1
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)

